=== modified file 'drizzled/function/get_system_var.cc'
--- drizzled/function/get_system_var.cc	2010-02-05 08:11:15 +0000
+++ drizzled/function/get_system_var.cc	2010-08-12 21:03:39 +0000
@@ -79,6 +79,7 @@
     my_error(ER_VARIABLE_IS_NOT_STRUCT, MYF(0), base_name->str);
     return 0;
   }
+  session->lex->setCacheable(false);
 
   set_if_smaller(component_name->length, (size_t)MAX_SYS_VAR_LENGTH);
 

=== modified file 'drizzled/message/include.am'
--- drizzled/message/include.am	2010-08-19 20:36:05 +0000
+++ drizzled/message/include.am	2010-08-24 16:45:09 +0000
@@ -48,12 +48,14 @@
 							drizzled/message/engine.pb.cc \
 							drizzled/message/transaction.pb.cc \
 							drizzled/message/schema.pb.cc \
+                                                        drizzled/message/resultset.pb.cc \
 							drizzled/message/table.pb.cc
 
 nobase_nodist_include_HEADERS+= \
 				drizzled/message/engine.pb.h \
 				drizzled/message/transaction.pb.h \
 				drizzled/message/schema.pb.h \
+                                drizzled/message/resultset.pb.h \
 				drizzled/message/table.pb.h 
 
 noinst_HEADERS+= \
@@ -93,12 +95,14 @@
 		${MESSAGE_DIR}/engine.proto \
 		${MESSAGE_DIR}/schema.proto \
 		${MESSAGE_DIR}/table.proto \
+                ${MESSAGE_DIR}/resultset.proto \
 		${MESSAGE_DIR}/transaction.proto
 
 BUILT_SOURCES += \
 		${MESSAGE_DIR}/engine.pb.h \
 		${MESSAGE_DIR}/schema.pb.h \
 		${MESSAGE_DIR}/table.pb.h \
+                ${MESSAGE_DIR}/resultset.pb.h \
 		${MESSAGE_DIR}/transaction.pb.h
 
 CLEANFILES += \
@@ -108,6 +112,8 @@
 	    ${MESSAGE_DIR}/schema.pb.cc \
 	    ${MESSAGE_DIR}/table.pb.h \
 	    ${MESSAGE_DIR}/table.pb.cc \
+            ${MESSAGE_DIR}/resultset.pb.h \
+            ${MESSAGE_DIR}/resultset.pb.cc \
 	    ${MESSAGE_DIR}/transaction.pb.h \
 	    ${MESSAGE_DIR}/transaction.pb.cc
 

=== added file 'drizzled/message/resultset.proto'
--- drizzled/message/resultset.proto	1970-01-01 00:00:00 +0000
+++ drizzled/message/resultset.proto	2010-08-15 04:17:26 +0000
@@ -0,0 +1,75 @@
+package drizzled.message;
+option optimize_for = SPEED;
+
+import "table.proto";
+import "schema.proto";
+
+
+/*
+ * Some minimal information transferred in the header of Statement
+ * submessage classes which identifies metadata about a specific
+ * field involved in a Statemet.
+ */
+message FieldMeta
+{
+  required string field_name = 1; /* Name of the field */
+  optional string field_alias = 2;
+  required string table_name = 3;
+  optional string table_alias = 4;
+  required string schema_name = 5;
+}
+
+/*
+ * Minimal information transferred in the header of Statement submessage
+ * classes which identifies metadata about the schema objects being
+ * modified in a Statement.
+ */
+message TableMeta
+{
+  required string schema_name = 1; /* Name of the containing schema */
+  required string table_name = 2; /* Name of the table */
+  optional string table_alias = 3; /* alias if defined */
+}
+
+
+/*
+ * Represents a single record being returned
+ *
+ * @note 
+ *
+ * A ResultSet contains one or more SelectRecord submessages, each
+ * of which represents a single record returned
+ */
+message SelectRecord
+{
+  repeated bytes record_value = 1;
+  repeated bool is_null = 2;
+}
+
+message SelectHeader
+{
+  repeated TableMeta table_meta = 1; /* Minimal metadata about the table affected */
+  repeated FieldMeta field_meta = 2; /* Collection of metadata about fields affected */
+}
+
+message SelectData
+{
+  required uint32 segment_id = 1; /* The segment number */
+  required bool end_segment = 2; /* Is this the final segment? */
+  repeated SelectRecord record = 3; /* The records inserted */
+}
+/*
+ * The message is composed the hash of the query, 
+ * a header (SelectHeader) containing metadata about:
+ * returned tables and fields.
+ * One or more data * segments (SelectData) containing the actual records
+ * being returned.
+ */
+message Resultset
+{
+  required string key= 1; /* contains a hashed value of: query + schema*/
+  required string schema= 2; /*current schema */
+  optional string sql = 3; /* May contain the original SQL string */
+  optional SelectHeader select_header = 4;
+  optional SelectData select_data = 5;
+}

=== modified file 'drizzled/plugin/query_cache.cc'
--- drizzled/plugin/query_cache.cc	2010-05-15 18:23:34 +0000
+++ drizzled/plugin/query_cache.cc	2010-08-20 05:59:13 +0000
@@ -2,6 +2,7 @@
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
  *  Copyright (C) 2008 Sun Microsystems
+ *  Copyright (C) 2010 Djellel Eddine Difallah
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,127 +33,150 @@
 
 namespace drizzled
 {
-
-vector<plugin::QueryCache *> all_query_cache;
+typedef vector<plugin::QueryCache *> QueryCaches;
+QueryCaches all_query_cache;
 
 /* Namespaces are here to prevent global symbol clashes with these classes */
 
-class TryFetchAndSendIterate
- : public unary_function<plugin::QueryCache *, bool>
-{
-  Session *session;
-  bool is_transactional;
-public:
-  TryFetchAndSendIterate(Session *session_arg, bool is_transactional_arg) :
-    unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), is_transactional(is_transactional_arg) { }
-
-  inline result_type operator()(argument_type handler)
-  {
-    if (handler->tryFetchAndSend(session, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("qcache plugin '%s' try_fetch_and_send() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
-  }
-};
-
-class SetIterate
- : public unary_function<plugin::QueryCache *, bool>
-{
-  Session *session;
-  bool is_transactional;
-public:
-  SetIterate(Session *session_arg, bool is_transactional_arg) :
-    unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), is_transactional(is_transactional_arg) { }
-
-  inline result_type operator()(argument_type handler)
-  {
-
-    if (handler->set(session, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR, _("qcache plugin '%s' set() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
-  }
-};
-
-class InvalidateTableIterate
- : public unary_function<plugin::QueryCache *, bool>
-{
-  Session *session;
-  bool is_transactional;
-public:
-  InvalidateTableIterate(Session *session_arg, bool is_transactional_arg) :
-    unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), is_transactional(is_transactional_arg) { }
-
-  inline result_type operator()(argument_type handler)
-  {
-
-    if (handler->invalidateTable(session, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("qcache plugin '%s' invalidateTable() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
-  }
-};
-
-
-class InvalidateDbIterate
- : public unary_function<plugin::QueryCache *, bool>
-{
-  Session *session;
-  const char *dbname;
-  bool is_transactional;
-public:
-  InvalidateDbIterate(Session *session_arg, const char *dbname_arg,
-                      bool is_transactional_arg) :
-    unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), dbname(dbname_arg),
-    is_transactional(is_transactional_arg) { }
-
-  inline result_type operator()(argument_type handler)
-  {
-    if (handler->invalidateDb(session, dbname, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("qcache plugin '%s' invalidateDb() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
-  }
-};
-
-class FlushIterate
- : public unary_function<plugin::QueryCache *, bool>
-{
-  Session *session;
-public:
-  FlushIterate(Session *session_arg) :
+class IsCachedIterate
+ : public unary_function<plugin::QueryCache *, bool>
+{
+  Session *session;
+public:
+  IsCachedIterate(Session* session_arg) :
+    unary_function<plugin::QueryCache *, bool>(),
+    session(session_arg) { }
+
+  inline result_type operator()(argument_type handler)
+  {
+    return handler->doIsCached(session);
+  }
+};
+
+bool plugin::QueryCache::isCached(Session *session)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            IsCachedIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+
+class SendCachedResultsetIterate
+ : public unary_function<plugin::QueryCache *, bool>
+{
+  Session *session;
+public:
+  SendCachedResultsetIterate(Session *session_arg) :
+    unary_function<plugin::QueryCache *, bool>(),
+    session(session_arg) { }
+
+  inline result_type operator()(argument_type handler)
+  {
+    return handler->doSendCachedResultset(session);
+  }
+};
+bool plugin::QueryCache::sendCachedResultset(Session *session)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            SendCachedResultsetIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+class PrepareResultsetIterate
+ : public unary_function<plugin::QueryCache *, bool>
+{
+  Session *session;
+public:
+  PrepareResultsetIterate(Session *session_arg) :
     unary_function<plugin::QueryCache *, bool>(), session(session_arg) { }
 
   inline result_type operator()(argument_type handler)
   {
-    if (handler->flush(session))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR, _("qcache plugin '%s' flush() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
-  }
-};
+    return handler->doPrepareResultset(session);
+  }
+};
+bool plugin::QueryCache::prepareResultset(Session *session)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            PrepareResultsetIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+class SetResultsetIterate
+ : public unary_function<plugin::QueryCache *, bool>
+{
+  Session *session;
+public:
+  SetResultsetIterate(Session *session_arg) :
+    unary_function<plugin::QueryCache *, bool>(),
+    session(session_arg) { }
+
+  inline result_type operator()(argument_type handler)
+  {
+    return handler->doSetResultset(session);
+  }
+};
+
+bool plugin::QueryCache::setResultset(Session *session)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            SetResultsetIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+class InsertRecordIterate
+ : public unary_function<plugin::QueryCache *, bool>
+{
+  Session *session;
+  List<Item> &item;
+public:
+  InsertRecordIterate(Session *session_arg, List<Item> &item_arg) :
+    unary_function<plugin::QueryCache *, bool>(),
+    session(session_arg), item(item_arg) { }
+
+  inline result_type operator()(argument_type handler)
+  {
+    return handler->doInsertRecord(session, item);
+  }
+};
+bool plugin::QueryCache::insertRecord(Session *session, List<Item> &items)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            InsertRecordIterate(session, items));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+
 
 bool plugin::QueryCache::addPlugin(plugin::QueryCache *handler)
 {
@@ -166,73 +190,4 @@
                         handler));
 }
 
-
-bool plugin::QueryCache::tryFetchAndSendDo(Session *session,
-                                           bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            TryFetchAndSendIterate(session, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::setDo(Session *session, bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            SetIterate(session, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::invalidateTableDo(Session *session,
-                                         bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            InvalidateTableIterate(session, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::invalidateDbDo(Session *session, const char *dbname,
-                                        bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            InvalidateDbIterate(session, dbname, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::flushDo(Session *session)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            FlushIterate(session));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
 } /* namespace drizzled */

=== modified file 'drizzled/plugin/query_cache.h'
--- drizzled/plugin/query_cache.h	2010-03-06 02:08:13 +0000
+++ drizzled/plugin/query_cache.h	2010-07-14 05:01:40 +0000
@@ -1,9 +1,8 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Definitions required for Query Cache plugin
- *
  *  Copyright (C) 2008 Sun Microsystems, Toru Maesaka
+ *  Copyright (C) 2010 Djellel Eddine Difallah
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,55 +21,60 @@
 #ifndef DRIZZLED_PLUGIN_QUERY_CACHE_H
 #define DRIZZLED_PLUGIN_QUERY_CACHE_H
 
+#include "drizzled/plugin.h"
 #include "drizzled/plugin/plugin.h"
+#include <drizzled/sql_list.h>
 
 namespace drizzled
 {
 class Session;
+class select_result;
 
 namespace plugin
 {
 
 /* 
   This is the API that a qcache plugin must implement.
-  it should implement each of these function pointers.
-  if a function pointer is NULL (not loaded), that's ok.
-
-  Return:
-    false = success
-    true  = failure
 */
+
 class QueryCache : public Plugin
 {
+private:  
+  
   QueryCache();
   QueryCache(const QueryCache &);
   QueryCache& operator=(const QueryCache &);
-public:
+
+public:  
+
   explicit QueryCache(std::string name_arg)
     : Plugin(name_arg, "QueryCache")
   {}
 
   virtual ~QueryCache() {}
-  /* Lookup the cache and transmit the data back to the client */
-  virtual bool tryFetchAndSend(Session *session,
-                               bool is_transactional)= 0;
-
-  virtual bool set(Session *session, bool is_transactional)= 0;
-  virtual bool invalidateTable(Session *session, bool is_transactional)= 0;
-  virtual bool invalidateDb(Session *session, const char *db_name,
-                            bool transactional)= 0;
-  virtual bool flush(Session *session)= 0;
+
+  /* these are the Query Cache interface functions */
+
+  /* Lookup the cache and transmit the data back to the client */
+  virtual bool doIsCached(Session* session)= 0;  
+  /* Lookup the cache and transmit the data back to the client */
+  virtual bool doSendCachedResultset(Session *session)= 0;
+  /* Send the current Resultset to the cache */
+  virtual bool doSetResultset(Session *session)= 0;
+  /* initiate a new Resultset (header) */
+  virtual bool doPrepareResultset(Session *session)= 0;
+  /* push a record to the current Resultset */
+  virtual bool doInsertRecord(Session *session, List<Item> &item)= 0;
 
   static bool addPlugin(QueryCache *handler);
   static void removePlugin(QueryCache *handler);
 
   /* These are the functions called by the rest of the Drizzle server */
-  static bool tryFetchAndSendDo(Session *session, bool transactional);
-  static bool setDo(Session *session, bool transactional);
-  static bool invalidateTableDo(Session *session, bool transactional);
-  static bool invalidateDbDo(Session *session, const char *db_name,
-                            bool transactional);
-  static bool flushDo(Session *session);
+  static bool isCached(Session *session);
+  static bool sendCachedResultset(Session *session);
+  static bool prepareResultset(Session *session);
+  static bool setResultset(Session *session);
+  static bool insertRecord(Session *session, List<Item> &item);
 };
 
 } /* namespace plugin */

=== modified file 'drizzled/select_send.h'
--- drizzled/select_send.h	2010-02-14 19:27:57 +0000
+++ drizzled/select_send.h	2010-07-14 05:01:40 +0000
@@ -22,6 +22,7 @@
 #define DRIZZLED_SELECT_SEND_H
 
 #include <drizzled/plugin/client.h>
+#include <drizzled/plugin/query_cache.h>
 #include <drizzled/plugin/transactional_storage_engine.h>
 
 namespace drizzled
@@ -111,6 +112,10 @@
         break;
       }
     }
+    /* Insert this record to the Resultset into the cache */
+    if (session->query_cache_key != "" && session->getResultsetMessage() != NULL)
+      plugin::QueryCache::insertRecord(session, items);
+
     session->sent_row_count++;
     if (session->is_error())
       return true;

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2010-08-21 22:26:34 +0000
+++ drizzled/session.cc	2010-08-24 16:45:09 +0000
@@ -212,7 +212,11 @@
   mysys_var= 0;
   scoreboard_index= -1;
   dbug_sentry=Session_SENTRY_MAGIC;
-  cleanup_done= abort_on_warning= no_warnings_for_error= false;
+  cleanup_done= abort_on_warning= no_warnings_for_error= false;  
+
+  /* query_cache init */
+  query_cache_key= "";
+  resultset= NULL;
 
   /* Variables with default values */
   proc_info="login";
@@ -1478,6 +1482,8 @@
 {
   /* Cleanup SQL processing state to reuse this statement in next query. */
   lex_end(lex);
+  query_cache_key= ""; // reset the cache key
+  resetResultsetMessage();
 }
 
 bool Session::copy_db_to(char **p_db, size_t *p_db_length)

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2010-08-17 01:34:55 +0000
+++ drizzled/session.h	2010-08-24 16:45:09 +0000
@@ -73,6 +73,7 @@
 {
 class Transaction;
 class Statement;
+class Resultset;
 }
 namespace internal
 {
@@ -393,7 +394,8 @@
     only responsible for freeing this member.
   */
   std::string db;
-
+  /* current cache key */
+  std::string query_cache_key;
   /**
     Constant for Session::where initialization in the beginning of every query.
 
@@ -1221,7 +1223,15 @@
   {
     return statement_message;
   }
-
+  
+  /**
+   * Returns a pointer to the current Resulset message for this
+   * Session, or NULL if no active message.
+   */
+  message::Resultset *getResultsetMessage() const
+  {
+    return resultset;
+  }
   /**
    * Sets the active transaction message used by the ReplicationServices
    * component.
@@ -1243,10 +1253,33 @@
   {
     statement_message= in_message;
   }
+
+  /**
+   * Sets the active Resultset message used by the Query Cache
+   * plugin.
+   *
+   * @param[in] Pointer to the message
+   */
+  void setResultsetMessage(message::Resultset *in_message)
+  {
+    resultset= in_message;
+  }
+  /**
+   * reset the active Resultset message used by the Query Cache
+   * plugin.
+   */
+
+  void resetResultsetMessage()
+  { 
+    resultset= NULL;
+  }
+
 private:
   /** Pointers to memory managed by the ReplicationServices component */
   message::Transaction *transaction_message;
   message::Statement *statement_message;
+  /* Pointer to the current resultset of Select query */
+  message::Resultset *resultset;
   plugin::EventObserverList *session_event_observers;
   
   /* Schema observers are mapped to databases. */

=== modified file 'drizzled/sql_lex.cc'
--- drizzled/sql_lex.cc	2010-07-27 18:52:50 +0000
+++ drizzled/sql_lex.cc	2010-08-12 21:03:39 +0000
@@ -276,6 +276,7 @@
   delete lex->result;
 
   lex->result= 0;
+  lex->setCacheable(true);
 
   if (lex->statement) 
     delete lex->statement;
@@ -1867,7 +1868,8 @@
     charset(NULL),
     sql_command(SQLCOM_END), 
     option_type(OPT_DEFAULT), 
-    is_lex_started(0)
+    is_lex_started(0),
+    cacheable(true)
 {
   reset_query_tables_list(true);
   statement= NULL;

=== modified file 'drizzled/sql_lex.h'
--- drizzled/sql_lex.h	2010-06-24 05:34:43 +0000
+++ drizzled/sql_lex.h	2010-08-12 21:03:39 +0000
@@ -931,6 +931,16 @@
     return false;
   }
   bool is_cross; // CROSS keyword was used
+  bool isCacheable()
+  {
+    return cacheable;
+  }
+  void setCacheable(bool val)
+  {
+    cacheable= val;
+  }
+private: 
+  bool cacheable;
 };
 
 extern void lex_start(Session *session);

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2010-08-12 17:21:13 +0000
+++ drizzled/sql_parse.cc	2010-08-24 16:45:09 +0000
@@ -44,6 +44,7 @@
 
 #include "drizzled/plugin/logging.h"
 #include "drizzled/plugin/query_rewrite.h"
+#include "drizzled/plugin/query_cache.h"
 #include "drizzled/plugin/authorization.h"
 #include "drizzled/optimizer/explain_plan.h"
 #include "drizzled/pthread_globals.h"
@@ -476,9 +477,7 @@
 
   /* now we are ready to execute the statement */
   res= lex->statement->execute();
-
   session->set_proc_info("query end");
-
   /*
     The return value for ROW_COUNT() is "implementation dependent" if the
     statement is not DELETE, INSERT or UPDATE, but -1 is what JDBC and ODBC
@@ -492,7 +491,6 @@
 
   return (res || session->is_error());
 }
-
 bool execute_sqlcom_select(Session *session, TableList *all_tables)
 {
   LEX	*lex= session->lex;
@@ -540,7 +538,13 @@
     {
       if (!result && !(result= new select_send()))
         return true;
+
+      /* Init the Query Cache plugin */
+      plugin::QueryCache::prepareResultset(session); 
       res= handle_select(session, lex, result, 0);
+      /* Send the Resultset to the cache */
+      plugin::QueryCache::setResultset(session); 
+
       if (result != lex->result)
         delete result;
     }
@@ -715,13 +719,22 @@
   uint64_t start_time= my_getsystime();
   lex_start(session);
   session->reset_for_next_command();
-
+  /* Check if the Query is Cached if and return true if yes
+   * TODO the plugin has to make sure that the query is cacheble
+   * by setting the query_safe_cache param to TRUE
+   */
+  bool res= true;
+  if (plugin::QueryCache::isCached(session))
+  {
+    res= plugin::QueryCache::sendCachedResultset(session);
+  }
+  if (not res)
+  {
+    return;
+  }
   LEX *lex= session->lex;
-
   Lex_input_stream lip(session, inBuf, length);
-
   bool err= parse_sql(session, &lip);
-
   if (!err)
   {
     {
@@ -731,9 +744,9 @@
                                  session->thread_id,
                                  const_cast<const char *>(session->db.empty() ? "" : session->db.c_str()));
         // Implement Views here --Brian
-
         /* Actually execute the query */
-        try {
+        try 
+        {
           mysql_execute_command(session);
         }
         catch (...)
@@ -749,7 +762,6 @@
   {
     assert(session->is_error());
   }
-
   lex->unit.cleanup();
   session->set_proc_info("freeing items");
   session->end_statement();

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2010-08-19 20:36:05 +0000
+++ drizzled/sql_yacc.yy	2010-08-24 16:45:09 +0000
@@ -3004,6 +3004,7 @@
             {
               DRIZZLE_YYABORT;
             }
+            Lex->setCacheable(false);
           }
         | DATE_SYM '(' expr ')'
           { $$= new (YYSession->mem_root) Item_date_typecast($3); }
@@ -3065,6 +3066,7 @@
             {
               DRIZZLE_YYABORT;
             }
+            Lex->setCacheable(false);
           }
         | YEAR_SYM '(' expr ')'
           { $$= new (YYSession->mem_root) Item_func_year($3); }
@@ -3092,6 +3094,7 @@
         | CURDATE optional_braces
           {
             $$= new (YYSession->mem_root) Item_func_curdate_local();
+            Lex->setCacheable(false);
           }
         | DATE_ADD_INTERVAL '(' expr ',' INTERVAL_SYM expr interval ')' %prec INTERVAL_SYM
           { $$= new (YYSession->mem_root) Item_date_add_interval($3,$6,$7,0); }
@@ -3102,10 +3105,12 @@
         | NOW_SYM optional_braces
           {
             $$= new (YYSession->mem_root) Item_func_now_local();
+            Lex->setCacheable(false);
           }
         | NOW_SYM '(' expr ')'
           {
             $$= new (YYSession->mem_root) Item_func_now_local($3);
+            Lex->setCacheable(false);
           }
         | POSITION_SYM '(' bit_expr IN_SYM expr ')'
           { $$ = new (YYSession->mem_root) Item_func_locate($5,$3); }
@@ -3163,9 +3168,15 @@
             }
           }
         | SYSDATE optional_braces
-          { $$= new (YYSession->mem_root) Item_func_sysdate_local(); }
+          { 
+            $$= new (YYSession->mem_root) Item_func_sysdate_local(); 
+            Lex->setCacheable(false);
+          }
         | SYSDATE '(' expr ')'
-          { $$= new (YYSession->mem_root) Item_func_sysdate_local($3); }
+          { 
+            $$= new (YYSession->mem_root) Item_func_sysdate_local($3); 
+            Lex->setCacheable(false);
+          }
         | TIMESTAMP_ADD '(' interval_time_stamp ',' expr ',' expr ')'
           { $$= new (YYSession->mem_root) Item_date_add_interval($7,$5,$3,0); }
         | TIMESTAMP_DIFF '(' interval_time_stamp ',' expr ',' expr ')'
@@ -3173,10 +3184,12 @@
         | UTC_DATE_SYM optional_braces
           {
             $$= new (YYSession->mem_root) Item_func_curdate_utc();
+            Lex->setCacheable(false);
           }
         | UTC_TIMESTAMP_SYM optional_braces
           {
             $$= new (YYSession->mem_root) Item_func_now_utc();
+            Lex->setCacheable(false);
           }
         ;
 
@@ -3197,6 +3210,7 @@
             {
               DRIZZLE_YYABORT;
             }
+            Lex->setCacheable(false);
 	  }
         | IF '(' expr ',' expr ',' expr ')'
           { $$= new (YYSession->mem_root) Item_func_if($3,$5,$7); }
@@ -3277,6 +3291,7 @@
             {
               DRIZZLE_YYABORT;
             }
+            Lex->setCacheable(false);
           }
         ;
 
@@ -3385,10 +3400,12 @@
           ident_or_text SET_VAR expr
           {
             $$= new Item_func_set_user_var($1, $3);
+            Lex->setCacheable(false);
           }
         | ident_or_text
           {
             $$= new Item_func_get_user_var(*YYSession, $1);
+            Lex->setCacheable(false);
           }
         | '@' opt_var_ident_type ident_or_text opt_component
           {
@@ -4356,6 +4373,7 @@
           OUTFILE TEXT_STRING_filesystem
           {
             LEX *lex= Lex;
+            lex->setCacheable(false);
             if (!(lex->exchange= new file_exchange($2.str, 0)) ||
                 !(lex->result= new select_export(lex->exchange)))
               DRIZZLE_YYABORT;
@@ -4366,6 +4384,7 @@
             LEX *lex=Lex;
             if (!lex->describe)
             {
+              lex->setCacheable(false);
               if (!(lex->exchange= new file_exchange($2.str,1)))
                 DRIZZLE_YYABORT;
               if (!(lex->result= new select_dump(lex->exchange)))
@@ -4373,7 +4392,7 @@
             }
           }
         | select_var_list_init
-          { }
+          {Lex->setCacheable(false);}
         ;
 
 /*

=== modified file 'plugin/memcached_functions/start_mc.sh.in'
--- plugin/memcached_functions/start_mc.sh.in	2009-12-19 19:09:29 +0000
+++ plugin/memcached_functions/start_mc.sh.in	2010-08-12 21:03:39 +0000
@@ -11,7 +11,7 @@
 
 startup()
 {
-  @MEMCACHED_BINARY@ -d -p $PORT -m $CACHESIZE -c $MAXCONN -U 0 -P /tmp/memc.pid.$PORT
+  memcached -d -p $PORT -m $CACHESIZE -c $MAXCONN -U 0 -P /tmp/memc.pid.$PORT
 }
 
 shutdown()

=== added directory 'plugin/memcached_query_cache'
=== added file 'plugin/memcached_query_cache/data_dictionary_schema.cc'
--- plugin/memcached_query_cache/data_dictionary_schema.cc	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/data_dictionary_schema.cc	2010-08-15 04:17:26 +0000
@@ -0,0 +1,133 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+
+/**
+ * @file
+ *
+ * Implements the DATA_DICTIONARY views which allows querying the
+ * state of the Query Cache
+ *
+ * CREATE TABLE DATA_DICTIONARY.QUERY_CACHE_ENTRIES (
+ *   KEY VARCHAR NOT NULL
+ * , SCHEMA VARCHAR NOT NULL
+ * , SQL VARCHAR NOT NULL
+ * );
+ */
+
+#include "config.h"
+
+#include "query_cache_service.h"
+#include "data_dictionary_schema.h"
+
+#include <fcntl.h>
+#include <sys/stat.h>
+
+using namespace std;
+using namespace drizzled;
+
+/*
+ *
+ * Query_Cache_Meta_ENTRIES view
+ *
+ */
+
+QueryCacheTool::QueryCacheTool() :
+  plugin::TableFunction("DATA_DICTIONARY", "QUERY_CACHE_ENTRIES")
+{
+  add_field("key");
+  add_field("schema");
+  add_field("sql");
+}
+
+QueryCacheTool::Generator::Generator(Field **arg) :
+  plugin::TableFunction::Generator(arg)
+{
+  it= QueryCacheService::cache.begin();
+  end= QueryCacheService::cache.end(); 
+}
+
+bool QueryCacheTool::Generator::populate()
+{
+  if (it == end)
+  { 
+    return false;
+  } 
+
+  QueryCacheService::CacheEntry &entry= *it;
+
+  push(entry.first);
+  push(entry.second.schema());
+  push(entry.second.sql());
+
+  it++;
+
+  return true;
+}
+
+/*
+ *
+ * Query_Cache_Cached_Tables view
+ *
+ */
+
+CachedTables::CachedTables() :
+  plugin::TableFunction("DATA_DICTIONARY", "QUERY_CACHED_TABLES")
+{
+  add_field("Table");
+  add_field("Cache_Keys");
+}
+
+CachedTables::Generator::Generator(Field **arg) :
+  plugin::TableFunction::Generator(arg)
+{
+  it= QueryCacheService::cachedTables.begin();
+  end= QueryCacheService::cachedTables.end(); 
+}
+
+bool CachedTables::Generator::populate()
+{
+  if (it == end)
+  { 
+    return false;
+  } 
+
+  QueryCacheService::CachedTablesEntry &entry= *it;
+
+  push(entry.first);
+  string list_keys;
+  vector<string>::iterator tmp;
+  for(tmp= entry.second.begin(); tmp != entry.second.end(); tmp++)
+  {
+    list_keys+= "::"+ *tmp;
+  }
+  push(list_keys);
+  it++;
+  return true;
+}

=== added file 'plugin/memcached_query_cache/data_dictionary_schema.h'
--- plugin/memcached_query_cache/data_dictionary_schema.h	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/data_dictionary_schema.h	2010-08-15 04:17:26 +0000
@@ -0,0 +1,110 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#ifndef PLUGIN_MEMCACHED_QUERY_CACHE_DATA_DICTIONARY_SCHEMA_H
+#define PLUGIN_MEMCACHED_QUERY_CACHE_DATA_DICTIONARY_SCHEMA_H
+
+#include "drizzled/plugin/table_function.h"
+#include "drizzled/field.h"
+
+namespace drizzled
+{
+class QueryCacheService;
+
+class QueryCacheTool : public drizzled::plugin::TableFunction
+{
+public:
+
+  QueryCacheTool();
+
+  class Generator : public drizzled::plugin::TableFunction::Generator
+  {
+  public:
+    Generator(drizzled::Field **arg);
+    bool populate();
+  private:
+    QueryCacheService::CacheEntries::iterator it;
+    QueryCacheService::CacheEntries::iterator end;
+  };
+
+  Generator *generator(drizzled::Field **arg)
+  {
+    return new Generator(arg);
+  }
+};
+
+class CachedTables : public drizzled::plugin::TableFunction
+{
+public:
+
+  CachedTables();
+
+  class Generator : public drizzled::plugin::TableFunction::Generator
+  {
+  public:
+    Generator(drizzled::Field **arg);
+    bool populate();
+  private:
+    QueryCacheService::CachedTablesEntries::iterator it;
+    QueryCacheService::CachedTablesEntries::iterator end;
+  };
+
+  Generator *generator(drizzled::Field **arg)
+  {
+    return new Generator(arg);
+  }
+};
+
+class QueryCacheStatusTool : public drizzled::plugin::TableFunction
+{
+public:
+  QueryCacheStatusTool() :
+    plugin::TableFunction("DATA_DICTIONARY", "QUERY_CACHE_STATUS")
+  {
+    add_field("VARIABLE_NAME");
+    add_field("VARIABLE_VALUE");
+  }
+
+  class Generator : public drizzled::plugin::TableFunction::Generator 
+  {
+    drizzled::drizzle_sys_var **status_var_ptr;
+
+  public:
+    Generator(drizzled::Field **fields);
+
+    bool populate();
+  };
+
+  Generator *generator(drizzled::Field **arg)
+  {
+    return new Generator(arg);
+  }
+};
+}
+#endif /* PLUGIN_MEMCACHED_QUERY_CACHE_DATA_DICTIONARY_SCHEMA_H */

=== added file 'plugin/memcached_query_cache/invalidator.cc'
--- plugin/memcached_query_cache/invalidator.cc	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/invalidator.cc	2010-08-15 04:17:26 +0000
@@ -0,0 +1,188 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#include "config.h"
+#include "invalidator.h"
+#include "query_cache_service.h"
+#include "memcached_qc.h"
+
+#include <drizzled/session.h>
+#include <drizzled/message/transaction.pb.h>
+#include <drizzled/message/table.pb.h>
+#include <drizzled/message/statement_transform.h>
+
+#include <vector>
+#include <string>
+#include <algorithm>
+
+
+using namespace drizzled;
+using namespace std;
+
+Invalidator::Invalidator(string name_arg)
+  : 
+    plugin::TransactionApplier(name_arg)
+{
+}
+
+plugin::ReplicationReturnCode
+Invalidator::apply(Session &in_session, 
+                   const message::Transaction &to_apply)
+{
+  (void) in_session;
+  string schema_name;
+  string table_name;
+
+  size_t stmt_size= to_apply.statement_size();
+
+  for (size_t i= 0; i < stmt_size; i++)
+  {
+    schema_name.clear();
+    table_name.clear();
+    const message::Statement &stmt= to_apply.statement(i);
+
+    /*
+     * We don't handle raw SQL for now.
+     */
+    if (stmt.type() != message::Statement::RAW_SQL)
+    {
+      parseStatementTableMetadata(stmt, schema_name, table_name);
+    }
+    else
+    {
+      continue; /* go on to the next statement */
+    }
+
+    /* Now lets invalidate all the entries of the table */
+    invalidateByTableName(schema_name, table_name);
+  }
+ return plugin::SUCCESS;
+}
+
+void Invalidator::parseStatementTableMetadata(const message::Statement &in_statement,
+                                                  string &in_schema_name,
+                                                  string &in_table_name) const
+{
+  switch (in_statement.type())
+  {
+    case message::Statement::INSERT:
+    {
+      const message::TableMetadata &metadata= in_statement.insert_header().table_metadata();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
+      break;
+    }
+    case message::Statement::UPDATE:
+    {
+      const message::TableMetadata &metadata= in_statement.update_header().table_metadata();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
+      break;
+    }
+    case message::Statement::DELETE:
+    {
+      const message::TableMetadata &metadata= in_statement.delete_header().table_metadata();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
+      break;
+    }
+    case message::Statement::CREATE_SCHEMA:
+    {
+      in_schema_name.assign(in_statement.create_schema_statement().schema().name());
+      in_table_name.clear();
+      break;
+    }
+    case message::Statement::ALTER_SCHEMA:
+    {
+      in_schema_name.assign(in_statement.alter_schema_statement().after().name());
+      in_table_name.clear();
+      break;
+    }
+    case message::Statement::DROP_SCHEMA:
+    {
+      in_schema_name.assign(in_statement.drop_schema_statement().schema_name());
+      in_table_name.clear();
+      break;
+    }
+    case message::Statement::CREATE_TABLE:
+    {
+      in_schema_name.assign(in_statement.create_table_statement().table().schema());
+      in_table_name.assign(in_statement.create_table_statement().table().name());
+      break;
+    }
+    case message::Statement::ALTER_TABLE:
+    {
+      in_schema_name.assign(in_statement.alter_table_statement().before().schema());
+      in_table_name.assign(in_statement.alter_table_statement().before().name());
+      break;
+    }
+    case message::Statement::DROP_TABLE:
+    {
+      const message::TableMetadata &metadata= in_statement.drop_table_statement().table_metadata();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
+      break;
+    }
+    default:
+    {
+      /* All other types have no schema and table information */
+      in_schema_name.clear();
+      in_table_name.clear();
+      break;
+    }
+  }  
+}
+void Invalidator::invalidateByTableName(const std::string &in_schema_name,
+                                        const std::string &in_table_name) const
+{
+  /* Reconstitute the schema+table key */
+  string key= in_schema_name+in_table_name;
+ 
+  /* Lookup for the invalidated table in the cached tables map */
+  QueryCacheService::CachedTablesEntries::iterator itt= QueryCacheService::cachedTables.find(key);
+  if (itt != QueryCacheService::cachedTables.end())
+  {
+    /* Extract the invloved hashes from the map and lookup the local Cache*/
+    QueryCacheService::CachedTablesEntry &entry= *itt;
+    vector<string>::iterator hash;
+    for(hash= entry.second.begin(); hash != entry.second.end(); hash++)
+    {
+      QueryCacheService::CacheEntries::iterator it= QueryCacheService::cache.find(*hash);
+      if (it != QueryCacheService::cache.end())
+      {
+        /* Remove the Query from the local Cache */
+        QueryCacheService::cache.erase(*hash);
+        /* Remove the Query from Memcached immediatly */
+        MemcachedQueryCache::getClient()->remove(*hash, 0);
+      }
+    }
+    /* finaly remove the table from the cached table list */
+    QueryCacheService::cachedTables.erase(key);
+  } 
+}

=== added file 'plugin/memcached_query_cache/invalidator.h'
--- plugin/memcached_query_cache/invalidator.h	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/invalidator.h	2010-08-16 22:04:48 +0000
@@ -0,0 +1,93 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#ifndef PLUGIN_MEMCACHED_QUERY_CACHE_INVALIDATOR_H
+#define PLUGIN_MEMCACHED_QUERY_CACHE_INVALIDATOR_H
+
+#include <drizzled/replication_services.h>
+
+#include <drizzled/plugin/transaction_replicator.h>
+#include <drizzled/plugin/transaction_applier.h>
+#include <drizzled/message/transaction.pb.h>
+
+#include <string>
+
+namespace drizzled
+{
+  class Session;
+}
+class Invalidator : public drizzled::plugin::TransactionApplier 
+{
+public:
+
+  Invalidator(std::string name_arg);
+
+  /** Destructor */
+  ~Invalidator() {}
+
+  /**
+   * Does something with the transaction message
+   *
+   * @param transaction message to be invalidated if present in the cache
+   */
+  drizzled::plugin::ReplicationReturnCode
+  apply(drizzled::Session &in_session,
+        const drizzled::message::Transaction &to_apply);
+  
+
+private:
+
+  /**
+   * Given a supplied Statement message, parse out the table
+   * and schema name from the various metadata and header 
+   * pieces for the different Statement types.
+   *
+   * @param[in] Statement to parse
+   * @param[out] Schema name
+   * @param[out] Table name
+   */
+  void parseStatementTableMetadata(const drizzled::message::Statement &in_statement,
+                                   std::string &in_schema_name,
+                                   std::string &in_table_name) const;
+
+  /**
+   * Given a schema name and table name, delete all the entries
+   * of the table from the cache.
+   *
+   * @param[in] stmt The statement message
+   * @param[in] Schema name
+   * @param[in] Table name
+   */
+  void invalidateByTableName(const std::string &in_schema_name,
+                             const std::string &in_table_name) const;
+
+
+};
+
+#endif /* PLUGIN_MEMCACHED_QUERY_CACHE_INVALIDATOR_H */

=== added file 'plugin/memcached_query_cache/memcached_qc.cc'
--- plugin/memcached_query_cache/memcached_qc.cc	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/memcached_qc.cc	2010-08-16 22:04:48 +0000
@@ -0,0 +1,487 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+
+
+#include "config.h"
+
+#include "drizzled/plugin.h"
+#include "drizzled/session.h"
+#include "drizzled/select_send.h"
+#include "drizzled/item/null.h"
+
+#include <gcrypt.h>
+#include <string>
+#include <iostream>
+#include <vector>
+
+#include "memcached_qc.h"
+#include "query_cache_udf_tools.h"
+#include "data_dictionary_schema.h"
+#include "invalidator.h"
+#include <boost/program_options.hpp>
+#include <drizzled/module/option_map.h>
+
+using namespace drizzled;
+using namespace std;
+namespace po= boost::program_options;
+
+static char* sysvar_memcached_servers= NULL;
+static ulong expiry_time;
+
+memcache::Memcache* MemcachedQueryCache::client;
+std::string MemcachedQueryCache::memcached_servers;
+
+static DRIZZLE_SessionVAR_BOOL(enable, 
+                               PLUGIN_VAR_SessionLOCAL,
+                               "Enable Memcached Query Cache",
+                               /* check_func */ NULL, 
+                               /* update_func */ NULL,
+                               /* default */ false);
+
+static int check_memc_servers(Session *,
+                              drizzle_sys_var *,
+                              void *save,
+                              drizzle_value *value)
+{
+  char buff[STRING_BUFFER_USUAL_SIZE];
+  int len= sizeof(buff);
+  const char *input= value->val_str(value, buff, &len);
+
+  if (input)
+  {
+    MemcachedQueryCache::setServers(input);
+    *(bool *) save= (bool) true;
+    return 0;
+  }
+
+  *(bool *) save= (bool) false;
+  return 1;
+}
+
+static void set_memc_servers(Session *,
+                             drizzle_sys_var *,
+                             void *var_ptr,
+                             const void *save)
+{
+  if (*(bool *) save != false)
+  {
+    *(const char **) var_ptr= MemcachedQueryCache::getServers();
+  }
+}
+
+static DRIZZLE_SYSVAR_STR(servers,
+                          sysvar_memcached_servers,
+                          PLUGIN_VAR_OPCMDARG,
+                          N_("List of memcached servers."),
+                          check_memc_servers, /* check func */
+                          set_memc_servers, /* update func */
+                          "127.0.0.1:11211"); /* default value */
+
+static DRIZZLE_SYSVAR_ULONG(expiry, 
+                            expiry_time,
+                            PLUGIN_VAR_OPCMDARG,
+                            "Expiry time of memcached entries",
+                             NULL, NULL, 1000, 0, ~0L, 0);
+
+bool MemcachedQueryCache::isSelect(string query)
+{
+  uint i= 0;
+  /*
+   Skip '(' characters in queries like following:
+   (select a from t1) union (select a from t1);
+  */
+  const char* sql= query.c_str();
+  while (sql[i] == '(')
+    i++;
+  /*
+   Test if the query is a SELECT
+   (pre-space is removed in dispatch_command).
+    First '/' looks like comment before command it is not
+    frequently appeared in real life, consequently we can
+    check all such queries, too.
+  */
+  if ((my_toupper(system_charset_info, sql[i])     != 'S' ||
+       my_toupper(system_charset_info, sql[i + 1]) != 'E' ||
+       my_toupper(system_charset_info, sql[i + 2]) != 'L') &&
+      sql[i] != '/')
+  {
+    return false;
+  }
+  return true;
+}
+
+bool MemcachedQueryCache::doIsCached(Session *session)
+{
+  if (SessionVAR(session, enable) && isSelect(session->query))
+  {
+    /* ToDo: Check against the cache content */
+    string query= session->query+session->db;
+    char* key= md5_key(query.c_str());
+    if(queryCacheService.isCached(key))
+    {
+     session->query_cache_key.assign(key);
+     free(key);
+     return true;
+    }
+    free(key);
+  }
+  return false;
+}
+
+bool MemcachedQueryCache::doSendCachedResultset(Session *session)
+{
+  /** TODO: pay attention to the case where the cache value is empty
+  * ie: there is a session in the process of caching the query
+  * and didn't finish the work
+  */ 
+  
+  LEX *lex= session->lex;
+  register Select_Lex *select_lex= &lex->select_lex;
+  select_result *result=lex->result;
+  if (not result && not (result= new select_send()))
+    return true;
+  result->prepare(select_lex->item_list, select_lex->master_unit());
+
+  /* fetching the resultset from memcached */  
+  vector<char> raw_resultset; 
+  getClient()->get(session->query_cache_key, raw_resultset);
+  if(raw_resultset.empty())
+    return false;
+  message::Resultset resultset_message;
+  if (not resultset_message.ParseFromString(string(raw_resultset.begin(),raw_resultset.end())))
+    return false;
+  List<Item> item_list;
+
+  /* Send the fields */
+  message::SelectHeader header= resultset_message.select_header();
+  size_t num_fields= header.field_meta_size();
+  for (size_t y= 0; y < num_fields; y++)
+  {
+    message::FieldMeta field= header.field_meta(y);
+    string value=field.field_alias();
+    item_list.push_back(new Item_string(value.c_str(), value.length(), system_charset_info));
+  }
+  result->send_fields(item_list);
+  item_list.empty();
+
+  /* Send the Data */
+  message::SelectData data= resultset_message.select_data();
+  session->limit_found_rows= 0; 
+  for (int j= 0; j < data.record_size(); j++)
+  {
+    message::SelectRecord record= data.record(j);
+    for (size_t y= 0; y < num_fields; y++)
+    {
+      if(record.is_null(y))
+      {
+        item_list.push_back(new Item_null());
+      }
+      else
+      {
+        string value=record.record_value(y);
+        item_list.push_back(new Item_string(value.c_str(), value.length(), system_charset_info));
+      }
+    }
+    result->send_data(item_list);
+    item_list.empty();
+  }
+  /* Send End of file */
+  result->send_eof();
+  /* reset the cache key at the session level */
+  session->query_cache_key= "";
+  return false;
+}
+
+/* Check if the tables in the query do not contain
+ * Data_dictionary
+ */
+void MemcachedQueryCache::checkTables(Session *session, TableList* in_table)
+{
+  for (TableList* tmp_table= in_table; tmp_table; tmp_table= tmp_table->next_global)
+  {
+    if (strcasecmp(tmp_table->db, "DATA_DICTIONARY") == 0)
+    {
+      session->lex->setCacheable(false);
+      break;
+    }
+  } 
+}
+
+/* init the current resultset in the session
+ * set the header message (hashkey= sql + schema)
+ */
+bool MemcachedQueryCache::doPrepareResultset(Session *session)
+{		
+  checkTables(session, session->lex->query_tables);
+  if (SessionVAR(session, enable) && session->lex->isCacheable())
+  {
+    /* Prepare and set the key for the session */
+    string query= session->query+session->db;
+    char* key= md5_key(query.c_str());
+
+    /* make sure only one thread will cache the query 
+     * if executed concurently
+     */
+    pthread_mutex_lock(&mutex);
+
+    if(not queryCacheService.isCached(key))
+    {
+      session->query_cache_key.assign(key);
+      free(key);
+    
+      /* create the Resultset */
+      message::Resultset *resultset= queryCacheService.setCurrentResultsetMessage(session);
+  
+      /* setting the resultset infos */
+      resultset->set_key(session->query_cache_key);
+      resultset->set_schema(session->db);
+      resultset->set_sql(session->query);
+      pthread_mutex_unlock(&mutex);
+      
+      return true;
+    }
+    pthread_mutex_unlock(&mutex);
+    free(key);
+  }
+  return false;
+}
+
+/* Send the current resultset to memcached
+ * Reset the current resultset of the session
+ */
+bool MemcachedQueryCache::doSetResultset(Session *session)
+{		
+  message::Resultset *resultset= session->getResultsetMessage();
+  if (SessionVAR(session, enable) && (not session->is_error()) && resultset != NULL && session->lex->isCacheable())
+  {
+    /* Generate the final Header */
+    queryCacheService.setResultsetHeader(*resultset, session, session->lex->query_tables);
+    /* serialize the Resultset Message */
+    std::string output;
+    resultset->SerializeToString(&output);
+
+    /* setting to memecahced */
+    time_t expiry= expiry_time;  // ToDo: add a user defined expiry
+    uint32_t flags= 0;
+    std::vector<char> raw(output.size());
+    memcpy(&raw[0], output.c_str(), output.size());
+    if(not client->set(session->query_cache_key, raw, expiry, flags))
+    {
+      delete resultset;
+      session->resetResultsetMessage();
+      return false;
+    }
+    
+    /* Clear the Selectdata from the Resultset to be localy cached
+     * Comment if Keeping the data in the header is needed
+     */
+    resultset->clear_select_data();
+
+    /* add the Resultset (including the header) to the hash 
+     * This is done after the memcached set
+     */
+    queryCacheService.cache[session->query_cache_key]= *resultset;
+
+    /* endup the current statement */
+    delete resultset;
+    session->resetResultsetMessage();
+    return true;
+  }
+  return false;
+}
+
+/* Adds a record (List<Item>) to the current Resultset.SelectData
+ */
+bool MemcachedQueryCache::doInsertRecord(Session *session, List<Item> &list)
+{		
+  if(SessionVAR(session, enable))
+  {
+    queryCacheService.addRecord(session, list);
+    return true;
+  }
+  return false;
+}
+
+char* MemcachedQueryCache::md5_key(const char *str)
+{
+  int msg_len= strlen(str);
+  /* Length of resulting sha1 hash - gcry_md_get_algo_dlen
+  * returns digest lenght for an algo */
+  int hash_len= gcry_md_get_algo_dlen( GCRY_MD_MD5 );
+  /* output sha1 hash - this will be binary data */
+  unsigned char* hash= (unsigned char*) malloc(hash_len);
+  /* output sha1 hash - converted to hex representation
+  * 2 hex digits for every byte + 1 for trailing \0 */
+  char *out= (char *) malloc( sizeof(char) * ((hash_len*2)+1) );
+  char *p= out;
+  /* calculate the SHA1 digest. This is a bit of a shortcut function
+  * most gcrypt operations require the creation of a handle, etc. */
+  gcry_md_hash_buffer( GCRY_MD_MD5, hash, str , msg_len );
+  /* Convert each byte to its 2 digit ascii
+  * hex representation and place in out */
+  int i;
+  for ( i = 0; i < hash_len; i++, p += 2 )
+  {
+    snprintf ( p, 3, "%02x", hash[i] );
+  }
+  free(hash);
+  return out;
+}
+
+/** User Defined Function print_query_cache_meta **/
+extern plugin::Create_function<PrintQueryCacheMetaFunction> *print_query_cache_meta_func_factory;
+plugin::Create_function<QueryCacheFlushFunction> *query_cache_flush_func= NULL;
+
+/** DATA_DICTIONARY views */
+static QueryCacheTool *query_cache_tool;
+static QueryCacheStatusTool *query_cache_status;
+static CachedTables *query_cached_tables;
+
+static int init(module::Context &context)
+{
+  const module::option_map &vm= context.getOptions();
+
+  if (vm.count("expiry"))
+  { 
+    if (expiry_time > (ulong)~0L)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value of expiry\n"));
+      exit(-1);
+    }
+  }
+
+  if (vm.count("servers"))
+  {
+    sysvar_memcached_servers= const_cast<char *>(vm["servers"].as<string>().c_str());
+  }
+
+  if (vm.count("enable"))
+  {
+    (SessionVAR(NULL,enable))= vm["enable"].as<bool>();
+  }
+
+  MemcachedQueryCache* memc= new MemcachedQueryCache("Memcached_Query_Cache", sysvar_memcached_servers);
+  context.add(memc);
+
+  Invalidator* invalidator= new Invalidator("Memcached_Query_Cache_Invalidator");
+  context.add(invalidator);
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  string replicator_name("default_replicator");
+  replication_services.attachApplier(invalidator, replicator_name);
+  
+  /* Setup the module's UDFs */
+  print_query_cache_meta_func_factory=
+    new plugin::Create_function<PrintQueryCacheMetaFunction>("print_query_cache_meta");
+  context.add(print_query_cache_meta_func_factory);
+  
+  query_cache_flush_func= new plugin::Create_function<QueryCacheFlushFunction>("query_cache_flush");
+  context.add(query_cache_flush_func);
+
+  /* Setup the module Data dict and status infos */
+  query_cache_tool= new (nothrow) QueryCacheTool();
+  context.add(query_cache_tool);
+  query_cache_status= new (nothrow) QueryCacheStatusTool();
+  context.add(query_cache_status);
+  query_cached_tables= new (nothrow) CachedTables();
+  context.add(query_cached_tables);
+  
+  return 0;
+}
+
+static drizzle_sys_var* vars[]= {
+  DRIZZLE_SYSVAR(enable),
+  DRIZZLE_SYSVAR(servers),
+  DRIZZLE_SYSVAR(expiry),
+  NULL
+};
+
+QueryCacheStatusTool::Generator::Generator(drizzled::Field **fields) :
+  plugin::TableFunction::Generator(fields)
+{ 
+  status_var_ptr= vars;
+}
+
+bool QueryCacheStatusTool::Generator::populate()
+{
+  if (*status_var_ptr)
+  {
+    std::ostringstream oss;
+    string return_value;
+
+    /* VARIABLE_NAME */
+    push((*status_var_ptr)->name);
+    if (strcmp((**status_var_ptr).name, "enable") == 0)
+      return_value= SessionVAR(&(getSession()), enable) ? "ON" : "OFF";
+    if (strcmp((**status_var_ptr).name, "servers") == 0) 
+      return_value= MemcachedQueryCache::getServers();
+    if (strcmp((**status_var_ptr).name, "expiry") == 0)
+    {
+      oss << expiry_time;
+      return_value= oss.str();
+    }
+    /* VARIABLE_VALUE */
+    if (return_value.length())
+      push(return_value);
+    else 
+      push(" ");
+
+    status_var_ptr++;
+
+    return true;
+  }
+  return false;
+}
+
+static void init_options(drizzled::module::option_context &context)
+{
+  context("servers",
+          po::value<string>()->default_value("127.0.0.1:11211"),
+          N_("List of memcached servers."));
+  context("expiry",
+          po::value<ulong>(&expiry_time)->default_value(1000),
+          N_("Expiry time of memcached entries"));
+  context("enable",
+          po::value<bool>()->default_value(false)->zero_tokens(),
+          N_("Enable Memcached Query Cache"));
+}
+
+DRIZZLE_DECLARE_PLUGIN
+{
+  DRIZZLE_VERSION_ID,
+  "Query_Cache",
+  "0.3",
+  "Djellel Eddine Difallah",
+  "Caches Select resultsets in Memcached",
+  PLUGIN_LICENSE_BSD,
+  init,   /* Plugin Init      */
+  vars, /* system variables */
+  init_options    /* config options   */
+}
+DRIZZLE_DECLARE_PLUGIN_END;

=== added file 'plugin/memcached_query_cache/memcached_qc.h'
--- plugin/memcached_query_cache/memcached_qc.h	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/memcached_qc.h	2010-08-16 22:04:48 +0000
@@ -0,0 +1,88 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+#ifndef PLUGIN_MEMCACHED_QUERY_CACHE_MEMCACHED_QC_H
+#define PLUGIN_MEMCACHED_QUERY_CACHE_MEMCACHED_QC_H
+
+#include "drizzled/plugin/query_cache.h"
+#include "query_cache_service.h"
+#include "drizzled/atomics.h"
+#include <libmemcached/memcached.hpp>
+
+namespace drizzled 
+{
+  class Session;
+  class select_result;
+  class String;
+  class TableList;
+}
+class MemcachedQueryCache : public drizzled::plugin::QueryCache
+{
+private:
+  pthread_mutex_t mutex;  
+  drizzled::QueryCacheService queryCacheService;
+  static memcache::Memcache* client;
+  static std::string memcached_servers;
+
+public:
+  
+  explicit MemcachedQueryCache(std::string name_arg, const std::string &servers_arg): drizzled::plugin::QueryCache(name_arg)
+  {
+    client= new memcache::Memcache(servers_arg);
+    pthread_mutex_init(&mutex, NULL);
+    queryCacheService= drizzled::QueryCacheService::singleton();
+  }
+  ~MemcachedQueryCache()
+  {
+    delete client;
+    pthread_mutex_destroy(&mutex);
+  };
+  bool doIsCached(drizzled::Session *session);
+  bool doSendCachedResultset(drizzled::Session *session);
+  bool doPrepareResultset(drizzled::Session *session);
+  bool doInsertRecord(drizzled::Session *session, drizzled::List<drizzled::Item> &item);
+  bool doSetResultset(drizzled::Session *session);
+  char* md5_key(const char* str);
+  void checkTables(drizzled::Session *session, drizzled::TableList* in_table);
+  bool isSelect(std::string query);
+  static memcache::Memcache* getClient()
+  {
+    return client;
+  }
+  static const char *getServers()
+  {
+    return memcached_servers.c_str();
+  }
+  static void setServers(const std::string &server_list)
+  {
+    memcached_servers.assign(server_list);
+    //getClient()->setServers(server_list);
+  }
+
+};
+#endif /* PLUGIN_MEMCACHED_QUERY_CACHE_MEMCACHED_QC_H */

=== added file 'plugin/memcached_query_cache/plugin.ini'
--- plugin/memcached_query_cache/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/plugin.ini	2010-08-15 04:17:26 +0000
@@ -0,0 +1,10 @@
+[plugin]
+title=Query Cache
+author=Djellel Eddine Difallah
+description=Query Cache for Memcached
+load_by_default=no
+disabled=yes
+headers=memcached_qc.h query_cache_service.h query_cache_udf_tools.h data_dictionary_schema.h invalidator.h
+sources=memcached_qc.cc query_cache_service.cc query_cache_udf_tools.cc data_dictionary_schema.cc invalidator.cc
+build_conditional="${ac_cv_libmemcached}" = "yes" -a "x${MEMCACHED_BINARY}" != "xno"
+ldflags=${LTLIBMEMCACHED}

=== added file 'plugin/memcached_query_cache/query_cache_service.cc'
--- plugin/memcached_query_cache/query_cache_service.cc	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/query_cache_service.cc	2010-08-16 06:20:05 +0000
@@ -0,0 +1,148 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#include "config.h"
+#include "drizzled/session.h"
+#include "query_cache_service.h"
+#include "drizzled/table_list.h"
+
+using namespace std;
+
+namespace drizzled
+{
+
+QueryCacheService::CacheEntries QueryCacheService::cache;
+QueryCacheService::CachedTablesEntries QueryCacheService::cachedTables;
+
+message::Resultset *QueryCacheService::setCurrentResultsetMessage(Session *in_session)
+{
+  message::Resultset *resultset= in_session->getResultsetMessage();
+
+  if (unlikely(resultset == NULL))
+  {
+    /* Allocate and initialize a new Resultset message 
+     * for this Session object.  Session is responsible for
+     * deleting Resultset message when done with it.
+     */
+    resultset= new (nothrow) message::Resultset();
+    in_session->setResultsetMessage(resultset);
+
+    /* for Atomic purpose, reserve an entry for the following query
+     */
+    cache[in_session->query_cache_key]= message::Resultset();
+  }
+  return resultset;
+}
+
+void QueryCacheService::setResultsetHeader(message::Resultset &resultset,
+                                          Session *in_session,
+                                          TableList *in_table)
+{
+  /* Set up the Select header */
+  message::SelectHeader *header= resultset.mutable_select_header();
+  
+  (void) in_session;
+
+  /* Extract all the tables mentioned in the query and 
+   * add the metadata to the SelectHeader
+   */
+
+  for (TableList* tmp_table= in_table; tmp_table; tmp_table= tmp_table->next_global)
+  {
+    message::TableMeta *table_meta= header->add_table_meta();
+    table_meta->set_schema_name(tmp_table->db, strlen(tmp_table->db));
+    table_meta->set_table_name(tmp_table->table_name, strlen(tmp_table->table_name));
+    /* Populate the cached tables hash */
+    QueryCacheService::cachedTables[strcat(tmp_table->db, tmp_table->table_name)].push_back(in_session->query_cache_key);
+  } 
+
+  /* Extract the returned fields 
+   * and add the field data to the SelectHeader
+   */
+  List_iterator_fast<Item> it(in_session->lex->select_lex.item_list);
+  Item *item;
+  while ((item=it++))
+  {
+    SendField field;
+    item->make_field(&field);
+    
+    message::FieldMeta *field_meta= header->add_field_meta();
+    field_meta->set_field_name(field.col_name, strlen(field.col_name));    
+    field_meta->set_schema_name(field.db_name, strlen(field.db_name));
+    field_meta->set_table_name(field.table_name, strlen(field.table_name));
+    field_meta->set_field_alias(field.org_col_name, strlen(field.org_col_name));
+    field_meta->set_table_alias(field.org_table_name, strlen(field.org_table_name));
+   }
+}
+
+bool QueryCacheService::addRecord(Session *in_session, List<Item> &list)
+{
+  message::Resultset *resultset= in_session->getResultsetMessage();
+  
+  if (resultset != NULL)
+  {
+    message::SelectData *data= resultset->mutable_select_data();
+    data->set_segment_id(1);
+    data->set_end_segment(true);
+    message::SelectRecord *record= data->add_record();
+
+    List_iterator_fast<Item> li(list);
+    
+    char buff[MAX_FIELD_WIDTH];
+    String buffer(buff, sizeof(buff), &my_charset_bin);
+    
+    Item *current_field;
+    while ((current_field= li++))
+    {
+      if (current_field->is_null())
+      {
+        record->add_is_null(true);
+        record->add_record_value("", 0);
+      } 
+      else 
+      {
+        String *string_value= current_field->val_str(&buffer);
+        record->add_is_null(false);
+        record->add_record_value(string_value->c_ptr(), string_value->length());
+        string_value->free();
+      }
+    }
+    return false;
+  }
+  return true;
+}
+
+bool QueryCacheService::isCached(string query)
+{
+  CacheEntries::iterator it= QueryCacheService::cache.find(query);
+  if (it != QueryCacheService::cache.end())
+     return true;
+  return false;
+}
+} /* namespace drizzled */

=== added file 'plugin/memcached_query_cache/query_cache_service.h'
--- plugin/memcached_query_cache/query_cache_service.h	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/query_cache_service.h	2010-08-16 22:04:48 +0000
@@ -0,0 +1,104 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+#ifndef PLUGIN_MEMCACHED_QUERY_CACHE_QUERY_CACHE_SERVICE_H
+#define PLUGIN_MEMCACHED_QUERY_CACHE_QUERY_CACHE_SERVICE_H
+
+#include "drizzled/message/resultset.pb.h"
+#include <drizzled/sql_list.h>
+#include <map>
+#include <vector>
+
+namespace drizzled
+{
+class TableList;
+class Item;
+
+namespace message
+{
+class Resultset;
+}
+
+class QueryCacheService
+{
+public:
+
+  typedef std::map<std::string, drizzled::message::Resultset> CacheEntries;
+  typedef std::pair<const std::string, drizzled::message::Resultset> CacheEntry;
+  typedef std::map<std::string, std::vector<std::string>> CachedTablesEntries;
+  typedef std::pair<const std::string, std::vector<std::string>> CachedTablesEntry;
+
+  static const size_t DEFAULT_RECORD_SIZE= 100;
+  static CacheEntries cache;
+  static CachedTablesEntries cachedTables;
+
+  /**
+   * Singleton method
+   * Returns the singleton instance of QueryCacheService
+   */
+  static inline QueryCacheService &singleton()
+  {
+    static QueryCacheService query_cache_service;
+    return query_cache_service;
+  };
+
+  /**
+   * Method which returns the active Resultset message
+   * for the supplied Session.  If one is not found, a new Resultset
+   * message is allocated, initialized, and returned.
+   *
+   * @param The session processing the Select
+   */
+  drizzled::message::Resultset *setCurrentResultsetMessage(drizzled::Session *in_session);
+
+  /**
+   * Helper method which initializes the header message for
+   * a Resultset.
+   *
+   * @param[inout] Resultset message container to modify
+   * @param[in] Pointer to the Session doing the processing
+   * @param[in] Pointer to the Table being inserted into
+   */
+  void setResultsetHeader(drizzled::message::Resultset &resultset,
+                       drizzled::Session *in_session,
+                       drizzled::TableList *in_table);
+  /**
+   * Creates a new SelectRecord GPB message and pushes it to
+   * currrent Resultset.
+   *
+   * @param Pointer to the Session which has inserted a record
+   * @param Pointer to the List<Items> to add
+   */
+  bool addRecord(drizzled::Session *in_session, drizzled::List<drizzled::Item> &list);
+
+  /* cache fetching */
+  static bool isCached(std::string query);
+  
+}; 
+} /* namespace drizzled */
+#endif /* PLUGIN_MEMCACHED_QUERY_CACHE_QUERY_CACHE_SERVICE_H */

=== added file 'plugin/memcached_query_cache/query_cache_udf_tools.cc'
--- plugin/memcached_query_cache/query_cache_udf_tools.cc	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/query_cache_udf_tools.cc	2010-08-16 06:20:05 +0000
@@ -0,0 +1,141 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/**
+ * @file
+ *
+ * Implements the PRINT_QUERY_CACHE_META(key) and QUERY_CACHE_FLUSH(expiry_time) UDFs.
+ */
+
+#include "config.h"
+#include <drizzled/plugin/function.h>
+#include <drizzled/item/func.h>
+#include <drizzled/function/str/strfunc.h>
+#include <drizzled/error.h>
+#include "drizzled/internal/my_sys.h"
+#include "drizzled/charset.h"
+
+#include <fcntl.h>
+
+#include "query_cache_udf_tools.h"
+#include "query_cache_service.h"
+#include "memcached_qc.h"
+
+#include <drizzled/message/resultset.pb.h>
+#include <google/protobuf/io/zero_copy_stream.h>
+#include <google/protobuf/io/zero_copy_stream_impl.h>
+#include <google/protobuf/io/coded_stream.h>
+#include <google/protobuf/text_format.h>
+
+using namespace std;
+using namespace drizzled;
+using namespace google;
+
+
+plugin::Create_function<PrintQueryCacheMetaFunction> *print_query_cache_meta_func_factory= NULL;
+
+void PrintQueryCacheMetaFunction::fix_length_and_dec()
+{
+  max_length= 2 * 1024 * 1024; /* 2MB size limit seems ok... */
+  args[0]->collation.set(
+    get_charset_by_csname(args[0]->collation.collation->csname,
+                          MY_CS_BINSORT), DERIVATION_COERCIBLE);
+}
+
+String *PrintQueryCacheMetaFunction::val_str(String *str)
+{
+  assert(fixed == true);
+
+  String *key_arg= args[0]->val_str(str);
+
+  if (key_arg == NULL)
+  {
+    my_error(ER_INVALID_NULL_ARGUMENT, MYF(0), func_name());
+    null_value= true;
+    return NULL;
+  }
+
+  null_value= false;
+
+  map<string, message::Resultset>::iterator it= QueryCacheService::cache.find(String_to_std_string(*key_arg));
+
+  if (it == QueryCacheService::cache.end())
+  {
+    my_error(ER_INVALID_NULL_ARGUMENT, MYF(0), func_name());
+    null_value= true;  
+    return NULL;
+  }
+
+  message::Resultset resultset_message= it->second;
+
+  string resultset_text;
+  protobuf::TextFormat::PrintToString(resultset_message, &resultset_text);
+
+  if (str->alloc(resultset_text.length()))
+  {
+    null_value= true;
+    return NULL;
+  }
+
+  str->length(resultset_text.length());
+
+  strncpy(str->ptr(), resultset_text.c_str(), resultset_text.length());
+
+  return str;
+}
+
+int64_t QueryCacheFlushFunction::val_int()
+{
+  bool res;
+  time_t expiration= 0;
+  null_value= false;
+
+  if ((arg_count != 0 && arg_count != 1) ||!MemcachedQueryCache::getClient())
+  {
+    return 0;
+  }
+
+  if (arg_count == 1)
+  {
+    String *tmp_exp= args[0]->val_str(&value);
+
+    expiration= (time_t)atoi(tmp_exp->c_ptr());
+  }
+
+  res= MemcachedQueryCache::getClient()->flush(expiration);
+  QueryCacheService::cache.clear();
+  QueryCacheService::cachedTables.clear();
+
+  if (not res)
+  {
+    return 0;
+  }
+
+  return 1;
+}

=== added file 'plugin/memcached_query_cache/query_cache_udf_tools.h'
--- plugin/memcached_query_cache/query_cache_udf_tools.h	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/query_cache_udf_tools.h	2010-08-16 22:04:48 +0000
@@ -0,0 +1,99 @@
+/* 
+ * Copyright (c) 2010, Djellel Eddine Difallah
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Djellel Eddine Difallah nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/**
+ * @file
+ *
+ * Defines the PRINT_QUERY_CACHE_META(KEY) UDF
+ *
+ * @details
+ *
+ * PRINT_QUERY_CACHE_META(key);
+ *
+ * prints a text representation of the meta data associated
+ * to a key stored in the cache
+ */
+
+#ifndef PLUGIN_MEMCACHED_QUERY_CACHE_QUERY_CACHE_UDF_TOOLS_H
+#define PLUGIN_MEMCACHED_QUERY_CACHE_QUERY_CACHE_UDF_TOOLS_H
+
+#include <drizzled/item/func.h>
+#include <drizzled/function/str/strfunc.h>
+
+class PrintQueryCacheMetaFunction : public drizzled::Item_str_func
+{
+public:
+  PrintQueryCacheMetaFunction() : Item_str_func() {}
+  drizzled::String *val_str(drizzled::String*);
+
+  void fix_length_and_dec();
+
+  const char *func_name() const
+  {
+    return "print_query_cache_meta";
+  }
+
+  bool check_argument_count(int n)
+  {
+    return (n == 1);
+  }
+};
+class QueryCacheFlushFunction: public drizzled::Item_int_func
+{
+  drizzled::String value;
+  drizzled::String failure_buff;
+public:
+  QueryCacheFlushFunction()
+    :
+      Item_int_func(),
+      failure_buff("0", &drizzled::my_charset_bin)
+  {}
+
+  const char *func_name() const
+  {
+    return "query_cache_flush";
+  }
+
+  int64_t val_int();
+
+  void fix_length_and_dec()
+  {
+    max_length= 32;
+  }
+  /*
+   * query_cache_flush takes 0 or 1 arguments
+   */
+  bool check_argument_count(int n)
+  {
+    return ((n == 1)||(n == 0));
+  }
+
+};
+
+#endif /* PLUGIN_MEMCACHED_QUERY_CACHE_QUERY_CACHE_UDF_TOOLS_H */

=== added file 'plugin/memcached_query_cache/start_mc.sh'
--- plugin/memcached_query_cache/start_mc.sh	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/start_mc.sh	2010-08-12 21:03:39 +0000
@@ -0,0 +1,50 @@
+#!/usr/bin/env bash
+
+PORT=19191
+if [ $MC_PORT ]; then
+  PORT=$MC_PORT
+fi
+
+MAXCONN=1024
+CACHESIZE=1024
+OPTIONS=""
+
+startup()
+{
+  memcached -d -p $PORT -m $CACHESIZE -c $MAXCONN -U 0 -P /tmp/memc.pid.$PORT
+}
+
+shutdown()
+{
+  if [ -f /tmp/memc.pid.$PORT ]
+  then
+    kill -9 `cat /tmp/memc.pid.$PORT`
+    rm /tmp/memc.pid.$PORT
+  fi
+}
+
+restart()
+{
+  shutdown
+  startup
+}
+
+
+# See how we were called.
+case "$1" in
+  start)
+        startup
+        ;;
+  stop)
+        shutdown
+        ;;
+  restart|reload)
+        restart
+        ;;
+  *)
+        echo $"Usage: $0 {start|stop|restart|reload}"
+        exit 1
+esac
+
+exit $?
+

=== added directory 'plugin/memcached_query_cache/tests'
=== added directory 'plugin/memcached_query_cache/tests/r'
=== added file 'plugin/memcached_query_cache/tests/r/query_cache_basic.result'
--- plugin/memcached_query_cache/tests/r/query_cache_basic.result	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/r/query_cache_basic.result	2010-08-15 04:17:26 +0000
@@ -0,0 +1,55 @@
+DROP TABLE IF EXISTS `t1`;
+DROP TABLE IF EXISTS `t2`;
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+INSERT INTO `t1` (name, age) values ("Meriem", NULL);
+set query_cache_enable= ON;
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+4	Meriem	NULL
+select * from t1 table1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+4	Meriem	NULL
+select age from t1;
+age
+20
+2
+22
+NULL
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+schema	sql
+test	select * from t1
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1 table1";
+schema	sql
+test	select * from t1 table1
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select age from t1";
+schema	sql
+test	select age from t1
+DROP TABLE IF EXISTS `t2`;
+CREATE TABLE `t2` (id int auto_increment primary key) select * from `t1` where age > 20;
+select name, age from t2;
+name	age
+ali	22
+select * from t1, t2 where t1.name = t2.name;
+id	name	age	id	name	age
+3	ali	22	3	ali	22
+select t.table from data_dictionary.query_cached_tables t;
+table
+testt1
+testt2
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select name, age from t2
+test	select age from t1
+test	select * from t1 table1
+test	select * from t1, t2 where t1.name = t2.name
+test	select * from t1

=== added file 'plugin/memcached_query_cache/tests/r/query_cache_flush.result'
--- plugin/memcached_query_cache/tests/r/query_cache_flush.result	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/r/query_cache_flush.result	2010-08-12 21:03:39 +0000
@@ -0,0 +1,39 @@
+DROP TABLE IF EXISTS `t1`;
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+set query_cache_enable= ON;
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+schema	sql
+test	select * from t1
+select query_cache_flush();
+query_cache_flush()
+1
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+schema	sql
+select count(*) from data_dictionary.query_cache_entries d;
+count(*)
+0
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+select count(*) from data_dictionary.query_cache_entries d;
+count(*)
+1
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+schema	sql
+test	select * from t1

=== added file 'plugin/memcached_query_cache/tests/r/query_cache_invalidation.result'
--- plugin/memcached_query_cache/tests/r/query_cache_invalidation.result	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/r/query_cache_invalidation.result	2010-08-15 04:17:26 +0000
@@ -0,0 +1,105 @@
+DROP TABLE IF EXISTS `t1`;
+DROP TABLE IF EXISTS `t2`;
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+set query_cache_enable= on;
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+INSERT INTO `t1` (name, age) values ("Meriem", NULL);
+CREATE TABLE `t2` (id int auto_increment primary key) select * from `t1` where age > 20;
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+4	Meriem	NULL
+select * from t2;
+id	name	age
+3	ali	22
+select * from t1, t2 where t1.name = t2.name;
+id	name	age	id	name	age
+3	ali	22	3	ali	22
+select t.table from data_dictionary.query_cached_tables t;
+table
+testt1
+testt2
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select * from t2
+test	select * from t1, t2 where t1.name = t2.name
+test	select * from t1
+INSERT INTO `t1` (name, age) values ("Foued", 35);
+select t.table from data_dictionary.query_cached_tables t;
+table
+testt2
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select * from t2
+UPDATE `t2` set age= age + 4;
+select t.table from data_dictionary.query_cached_tables t;
+table
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+select query_cache_flush();
+query_cache_flush()
+1
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+4	Meriem	NULL
+5	Foued	35
+select * from t2;
+id	name	age
+3	ali	26
+select * from t1, t2 where t1.name = t2.name;
+id	name	age	id	name	age
+3	ali	22	3	ali	26
+select t.table from data_dictionary.query_cached_tables t;
+table
+testt1
+testt2
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select * from t2
+test	select * from t1, t2 where t1.name = t2.name
+test	select * from t1
+delete from t1 where name= "ali";
+select t.table from data_dictionary.query_cached_tables t;
+table
+testt2
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select * from t2
+select query_cache_flush();
+query_cache_flush()
+1
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+4	Meriem	NULL
+5	Foued	35
+select * from t2;
+id	name	age
+3	ali	26
+select * from t1, t2 where t1.name = t2.name;
+id	name	age	id	name	age
+select t.table from data_dictionary.query_cached_tables t;
+table
+testt1
+testt2
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select * from t2
+test	select * from t1, t2 where t1.name = t2.name
+test	select * from t1
+DROP TABLE `t2`;
+select t.table from data_dictionary.query_cached_tables t;
+table
+testt1
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select * from t1

=== added file 'plugin/memcached_query_cache/tests/r/query_cache_udf.result'
--- plugin/memcached_query_cache/tests/r/query_cache_udf.result	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/r/query_cache_udf.result	2010-08-15 04:17:26 +0000
@@ -0,0 +1,60 @@
+DROP TABLE IF EXISTS `t1`;
+DROP TABLE IF EXISTS `t2`;
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+set query_cache_enable= ON;
+select * from t1;
+id	name	age
+1	leila	20
+2	lockman	2
+3	ali	22
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+schema	sql
+test	select * from t1
+select d.key from data_dictionary.query_cache_entries d where d.sql like "select * from t1" into @key;;
+select print_query_cache_meta(@key);
+print_query_cache_meta(@key)
+key: "fd2025cd0ef840b16804c84d54327275"
+schema: "test"
+sql: "select * from t1"
+select_header {
+  table_meta {
+    schema_name: "test"
+    table_name: "t1"
+  }
+  field_meta {
+    field_name: "id"
+    field_alias: "id"
+    table_name: "t1"
+    table_alias: "t1"
+    schema_name: "test"
+  }
+  field_meta {
+    field_name: "name"
+    field_alias: "name"
+    table_name: "t1"
+    table_alias: "t1"
+    schema_name: "test"
+  }
+  field_meta {
+    field_name: "age"
+    field_alias: "age"
+    table_name: "t1"
+    table_alias: "t1"
+    schema_name: "test"
+  }
+}
+
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "%print_query_cache_meta%";
+schema	sql
+select query_cache_flush();
+query_cache_flush()
+1
+select count(*) from data_dictionary.query_cache_entries;
+count(*)
+0
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "%print_query_cache_meta%";
+schema	sql

=== added directory 'plugin/memcached_query_cache/tests/t'
=== added file 'plugin/memcached_query_cache/tests/t/query_cache_basic-master.opt'
--- plugin/memcached_query_cache/tests/t/query_cache_basic-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_basic-master.opt	2010-08-12 21:03:39 +0000
@@ -0,0 +1,1 @@
+--plugin_add=memcached_query_cache

=== added file 'plugin/memcached_query_cache/tests/t/query_cache_basic.test'
--- plugin/memcached_query_cache/tests/t/query_cache_basic.test	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_basic.test	2010-08-15 04:17:26 +0000
@@ -0,0 +1,43 @@
+# we are assuming the memcached binary is contained in PATH or
+# common directories we might expect it to be in
+--exec sh $TOP_BUILDDIR/plugin/memcached_query_cache/start_mc.sh stop
+--exec sh $TOP_BUILDDIR/plugin/memcached_query_cache/start_mc.sh start
+
+--disable_warnings
+DROP TABLE IF EXISTS `t1`;
+DROP TABLE IF EXISTS `t2`;
+--enable_warnings
+
+--replace_result $MC_PORT MC_A_PORT
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+INSERT INTO `t1` (name, age) values ("Meriem", NULL);
+
+set query_cache_enable= ON;
+# Test the cache 
+
+select * from t1;
+select * from t1 table1;
+select age from t1;
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1 table1";
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select age from t1";
+
+#test of the query cached tables structure
+
+--disable_warnings
+DROP TABLE IF EXISTS `t2`;
+--enable_warnings
+CREATE TABLE `t2` (id int auto_increment primary key) select * from `t1` where age > 20;
+
+select name, age from t2;
+select * from t1, t2 where t1.name = t2.name;
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+
+--exec sh $TOP_BUILDDIR/plugin/memcached_query_cache/start_mc.sh stop

=== added file 'plugin/memcached_query_cache/tests/t/query_cache_flush-master.opt'
--- plugin/memcached_query_cache/tests/t/query_cache_flush-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_flush-master.opt	2010-08-12 21:03:39 +0000
@@ -0,0 +1,1 @@
+--plugin_add=memcached_query_cache

=== added file 'plugin/memcached_query_cache/tests/t/query_cache_flush.test'
--- plugin/memcached_query_cache/tests/t/query_cache_flush.test	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_flush.test	2010-08-15 06:58:22 +0000
@@ -0,0 +1,31 @@
+# we are assuming the memcached binary is contained in PATH or
+# common directories we might expect it to be in
+--exec sh ../plugin/memcached_query_cache/start_mc.sh stop
+--exec sh ../plugin/memcached_query_cache/start_mc.sh start
+
+--disable_warnings
+DROP TABLE IF EXISTS `t1`;
+--enable_warnings
+
+--replace_result $MC_PORT MC_A_PORT
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+
+set query_cache_enable= ON;
+select * from t1;
+select * from t1;
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+select query_cache_flush();
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+select count(*) from data_dictionary.query_cache_entries d;
+select * from t1;
+select count(*) from data_dictionary.query_cache_entries d;
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "select * from t1";
+
+
+--exec sh  ../plugin/memcached_query_cache/start_mc.sh stop
+

=== added file 'plugin/memcached_query_cache/tests/t/query_cache_invalidation-master.opt'
--- plugin/memcached_query_cache/tests/t/query_cache_invalidation-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_invalidation-master.opt	2010-08-15 04:17:26 +0000
@@ -0,0 +1,1 @@
+--plugin_add=memcached_query_cache

=== added file 'plugin/memcached_query_cache/tests/t/query_cache_invalidation.test'
--- plugin/memcached_query_cache/tests/t/query_cache_invalidation.test	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_invalidation.test	2010-08-15 04:17:26 +0000
@@ -0,0 +1,76 @@
+# we are assuming the memcached binary is contained in PATH or
+# common directories we might expect it to be in
+--exec sh $TOP_BUILDDIR/plugin/memcached_query_cache/start_mc.sh stop
+--exec sh $TOP_BUILDDIR/plugin/memcached_query_cache/start_mc.sh start
+
+--disable_warnings
+DROP TABLE IF EXISTS `t1`;
+DROP TABLE IF EXISTS `t2`;
+--enable_warnings
+
+--replace_result $MC_PORT MC_A_PORT
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+set query_cache_enable= on;
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+INSERT INTO `t1` (name, age) values ("Meriem", NULL);
+
+CREATE TABLE `t2` (id int auto_increment primary key) select * from `t1` where age > 20;
+select * from t1;
+select * from t2;
+select * from t1, t2 where t1.name = t2.name;
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+
+#Test the invalidation BY
+
+#1: INSERT
+INSERT INTO `t1` (name, age) values ("Foued", 35);
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+
+#2: UPDATE
+UPDATE `t2` set age= age + 4;
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+
+
+#3: DELETE
+select query_cache_flush();
+select * from t1;
+select * from t2;
+select * from t1, t2 where t1.name = t2.name;
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+delete from t1 where name= "ali";
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+
+#4: ALTER (replication of this statement is not working)
+##select query_cache_flush();
+##select * from t1;
+##select * from t2;
+##select * from t1, t2 where t1.name = t2.name;
+##select t.table from data_dictionary.query_cached_tables t;
+##select d.schema, d.sql from data_dictionary.query_cache_entries d;
+##ALTER TABLE `t2` add column (education varchar(20) default "Phd");
+##select t.table from data_dictionary.query_cached_tables t;
+##select d.schema, d.sql from data_dictionary.query_cache_entries d;
+
+#5: DROP
+select query_cache_flush();
+select * from t1;
+select * from t2;
+select * from t1, t2 where t1.name = t2.name;
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+DROP TABLE `t2`;
+select t.table from data_dictionary.query_cached_tables t;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+
+
+--exec sh $TOP_BUILDDIR/plugin/memcached_query_cache/start_mc.sh stop
+

=== added file 'plugin/memcached_query_cache/tests/t/query_cache_udf-master.opt'
--- plugin/memcached_query_cache/tests/t/query_cache_udf-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_udf-master.opt	2010-08-12 21:03:39 +0000
@@ -0,0 +1,1 @@
+--plugin_add=memcached_query_cache

=== added file 'plugin/memcached_query_cache/tests/t/query_cache_udf.test'
--- plugin/memcached_query_cache/tests/t/query_cache_udf.test	1970-01-01 00:00:00 +0000
+++ plugin/memcached_query_cache/tests/t/query_cache_udf.test	2010-08-15 06:58:22 +0000
@@ -0,0 +1,30 @@
+# we are assuming the memcached binary is contained in PATH or
+# common directories we might expect it to be in
+--exec sh ../plugin/memcached_query_cache/start_mc.sh stop
+--exec sh ../plugin/memcached_query_cache/start_mc.sh start
+
+--disable_warnings
+DROP TABLE IF EXISTS `t1`;
+DROP TABLE IF EXISTS `t2`;
+--enable_warnings
+
+--replace_result $MC_PORT MC_A_PORT
+set global query_cache_servers= "127.0.0.1:$MC_PORT";
+
+CREATE TABLE `t1` (id integer NOT NULL auto_increment primary key, name varchar(20), age int);
+INSERT INTO `t1` (name, age) values ("leila", 20);
+INSERT INTO `t1` (name, age) values ("lockman", 2);
+INSERT INTO `t1` (name, age) values ("ali", 22);
+
+set query_cache_enable= ON;
+select * from t1;
+select d.schema, d.sql from data_dictionary.query_cache_entries d;
+--eval select d.key from data_dictionary.query_cache_entries d where d.sql like "select * from t1" into @key;
+# flush
+select print_query_cache_meta(@key);
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "%print_query_cache_meta%";
+select query_cache_flush();
+select count(*) from data_dictionary.query_cache_entries;
+select d.schema, d.sql from data_dictionary.query_cache_entries d where d.sql like "%print_query_cache_meta%";
+--exec sh  ../plugin/memcached_query_cache/start_mc.sh stop
+

=== modified file 'po/POTFILES.in'
--- po/POTFILES.in	2010-08-19 16:45:03 +0000
+++ po/POTFILES.in	2010-08-24 16:45:09 +0000
@@ -69,6 +69,7 @@
 plugin/logging_query/logging_query.cc
 plugin/logging_stats/logging_stats.cc
 plugin/md5/md5.cc
+plugin/memcached_query_cache/memcached_qc.cc
 plugin/memcached_stats/analysis_table.cc
 plugin/memcached_stats/memcached_stats.cc
 plugin/memcached_stats/stats_table.cc

